#!/usr/bin/perl

# Seppo - The Form Analysator - 0.1
# Copyright (C) 2002  Matti Lattu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

use CGI;
use CGI::Carp 'fatalsToBrowser';
use Fcntl 'LOCK_SH','LOCK_EX','LOCK_UN';

# Some servers (like IIS by default) do not chdir to the directory where
# the executed cgi lives (like Apache does). In these servers you might get
# errors about missing form.pm and template.pm modules. In this case
# you might want to define following path to the modules (usually your Seppo
# directory).

#use lib '/usr/local/www/cgi-bin/seppo';

use form;
use template;

# $CONFIG is used as prefix for all "system files" that are read-only
# for the web server, namely
#	FORMNAME.form (form definition)
#	tpl-FORMNAME.html (output templates)
# For maximal security use full path. Make sure that system files are
# not world-writable.

my $CONFIG="./";

# $DATA is used as prefix for all "data files" that are read/written by
# the web server. Such files are:
#	FORMNAME.raw (raw data, switch "save_data_raw")
#	FORMNAME.txt (analysed data, switch "save_data_analysed")
# For maximal security use full path. Make sure that the web server process
# has read/write access to these files. If you have users accessing the server
# locally (e.g. through ssh), you might not want to give them a read/write
# access for the files.
#
# NOTE: When installing new questionnaire where either raw or analysed
#       data is stored you have to create empty data files by hand!
#       This procedure is a feature that encourages you to check the file
#       permissions.

my $DATA="/pub/www/data/malu/seppo/";

# After writing form definition and output template you have configured
# your Seppo.

# Following characters are allowed in index and response formulas:
my $FORMULA_ALLOWED = '\w\s\+-\/\*\(\)\.\!<>\="';

my $ERROR_HEADER = "Content-type: text/plain\nPragma: no-cache\n\n";
my $OK_HEADER = "Content-type: text/html\nPragma: no-cache\n\n";

my $cgi = new CGI;

my $formname = $cgi->param('formname');
$formname =~ s/\W//g;

my $form = new form();


# Read configuration and set object accordingly

my %confdata = &readConfig($CONFIG.$formname.".form");
if (!(%confdata)) {
  print $ERROR_HEADER;
  print "Error: Form \"$formname\" has no valid attributes\n\n";
  exit 1;
  }
if ($confdata{error} ne '') {
  print $ERROR_HEADER;
  print "Error: $confdata{error}\n\n";
  exit 1;
  }

# You might want to comment this:
if ($confdata{warnings} ne '') {
  print $ERROR_HEADER;
  print "You form definition file casted following warnings:\n\n";
  foreach (split(/\t/,$confdata{warnings})) {
    print "$_\n";
    }
  exit 1;
  }
  
if (!$form->setConf(%confdata)) {
  die("setConf failed");
  }
		

# Store CGI input to object

my @cgi_vars = $cgi->param;

foreach (@cgi_vars) {
  if ($_ =~ /^q(\d+)$/i) {
    my $value = $cgi->param("q".$1);
    $value =~ s/\D//g;			# Allow numeric data only
    $form->setItem("q".$1 => $value);
    }
  }
  
# Process form
my %alltags = $form->getTagsAllSafe();

# Store raw data

if ($confdata{save_data_raw}) {
  if (!(-f "$DATA$formname.raw")) {
    die("Raw data save for form $formname: could not find $DATA$formname.raw");
    }
  open(F,">>$DATA$formname.raw") || die("Could not open $DATA$formname.raw for writing: $!");
  flock(F,LOCK_EX);
  seek(F,0,2);

  print F join("\t",time,$form->getItem())."\n";
    
  flock(F,LOCK_UN);
  close(F);
  }

# Store analysed data

if ($confdata{save_data_analysed} ne '') {
  if (!(-f "$DATA$formname.txt")) {
    die("Analysed data save for form $formname: could not find $DATA$formname.txt");
    }
  open(F,">>$DATA$formname.txt") || die("Could not open $DATA$formname.txt for writing: $!");
  flock(F,LOCK_EX);
  seek(F,0,2);

  print F join("\t",time,$form->getItemList($confdata{save_data_analysed}))."\n";
  
  flock(F,LOCK_UN);
  close(F);
  }

# Output information through template module

print $OK_HEADER;

my $tplfile = $CONFIG."tpl-".$formname.".html";
if (!(-f $tplfile)) {
  die("Template file $tplfile was not found");
  }
  
&template($tplfile,'stdout',%alltags);

exit;


sub readConfig {
 if ($_[0] eq '') {
   return ('error','Configuration file was undefined');
   }
 if (!(-f $_[0])) {
   return ('error',"Configuration file $_[0] was not found");
   }
 
 my %ret = ();
 
 open(IN,"<$_[0]");
 flock(IN,LOCK_SH);
 

 my $line;
 my $cmd;
 my $cmd_n;
 my $cmd_str;
 my $par;
 
 while ($line = <IN>) {
   chop($line);
   if ($line =~ /^#/) { next; }
   ($cmd,$par) = split(/\s/,$line,2);
   
   $cmd_n = $cmd;		# Numeric part of command
   $cmd_n =~ s/\D//g;
   
   $cmd_str = lc($cmd);		# String part of command
   $cmd_str =~ s/[^a-z_-]//g;

   if ($cmd_str eq '') { next; }
      
   my $cmd_ok = 0;
  
   if ($cmd_str eq 'item_name_') {
     $cmd_ok = 1;
     $par =~ s/[^\w\s]//g;
     $ret{"item_name".$cmd_n} = $par;
     }
   if ($cmd_str eq 'index_name_') {
     $cmd_ok = 1;
     $par =~ s/[^\w\s]//g;
     $ret{"index_name".$cmd_n} = $par;
     }
   if ($cmd_str eq 'index_formula_') {
     $cmd_ok = 1;
     $par =~ s/[^$FORMULA_ALLOWED]//g;
     $ret{"index_formula".$cmd_n} = $par;
     }
   if ($cmd_str eq 'response_') {
     $cmd_ok = 1;
     $par =~ s/\t/ /g;
     
     my $condition;
     my $response;
     
     if ($par =~ /.*"(.*?)".*"(.*?)".*/) {   # parse out values
       $condition = $1;
       $response = $2;

       $condition =~ s/[^$FORMULA_ALLOWED]//g;

       $ret{"response".$cmd_n} .= "$condition\x01$response\x02";
       }
     }
   if ($cmd_str eq 'save_data_raw') {
     $cmd_ok = 1;
     $ret{save_data_raw} = 1;
     }
   if ($cmd_str eq 'save_data_analysed') {
     $cmd_ok = 1;
     $par =~ s/[^\d\sqir]//gi;
     $ret{save_data_analysed} = join(" ",split(/\s/,$par));
     }

   if (!$cmd_ok) {
     $ret{warnings} = join("\t",split(/\t/,$ret{warnings}),"Unknown form definition tag: $cmd_str");
     }
   }
 
 flock(IN,LOCK_UN);
 close(IN);
 
 return (%ret);
 }
